Exception: Parallel::Break
- Inherits:
-
StandardError
- Object
- StandardError
- Parallel::Break
- Defined in:
- lib/parallel.rb
Direct Known Subclasses
Instance Attribute Summary collapse
-
#value ⇒ Object
readonly
Returns the value of attribute value.
Class Method Summary collapse
-
._load(data) ⇒ Object
marshal_load that is used for ruby exceptions.
Instance Method Summary collapse
-
#_dump(_depth) ⇒ Object
marshal_dump that is used for ruby exceptions avoid dumping the cause since nobody needs that and it can include undumpable exceptions.
-
#initialize(value = nil) ⇒ Break
constructor
A new instance of Break.
Constructor Details
#initialize(value = nil) ⇒ Break
Returns a new instance of Break.
15 16 17 18 |
# File 'lib/parallel.rb', line 15 def initialize(value = nil) super() @value = value end |
Instance Attribute Details
#value ⇒ Object (readonly)
Returns the value of attribute value.
13 14 15 |
# File 'lib/parallel.rb', line 13 def value @value end |
Class Method Details
._load(data) ⇒ Object
marshal_load that is used for ruby exceptions
27 28 29 |
# File 'lib/parallel.rb', line 27 def self._load(data) new(Marshal.load(data)) end |
Instance Method Details
#_dump(_depth) ⇒ Object
marshal_dump that is used for ruby exceptions avoid dumping the cause since nobody needs that and it can include undumpable exceptions
22 23 24 |
# File 'lib/parallel.rb', line 22 def _dump(_depth) Marshal.dump(@value) end |