Class: Parallel::Worker

Inherits:
Object
  • Object
show all
Defined in:
lib/parallel.rb

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(read, write, pid, serializer) ⇒ Worker

Returns a new instance of Worker.



67
68
69
70
71
72
# File 'lib/parallel.rb', line 67

def initialize(read, write, pid, serializer)
  @read = read
  @write = write
  @pid = pid
  @serializer = serializer
end

Instance Attribute Details

#pidObject (readonly)

Returns the value of attribute pid.



64
65
66
# File 'lib/parallel.rb', line 64

def pid
  @pid
end

#readObject (readonly)

Returns the value of attribute read.



64
65
66
# File 'lib/parallel.rb', line 64

def read
  @read
end

#threadObject

Returns the value of attribute thread.



65
66
67
# File 'lib/parallel.rb', line 65

def thread
  @thread
end

#writeObject (readonly)

Returns the value of attribute write.



64
65
66
# File 'lib/parallel.rb', line 64

def write
  @write
end

Instance Method Details

#close_pipesObject

might be passed to started_processes and simultaneously closed by another thread when running in isolation mode, so we have to check if it is closed before closing



81
82
83
84
# File 'lib/parallel.rb', line 81

def close_pipes
  read.close unless read.closed?
  write.close unless write.closed?
end

#stopObject



74
75
76
77
# File 'lib/parallel.rb', line 74

def stop
  close_pipes
  wait # if it goes zombie, rather wait here to be able to debug
end

#work(data) ⇒ Object



86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
# File 'lib/parallel.rb', line 86

def work(data)
  begin
    @serializer.dump(data, write)
  rescue Errno::EPIPE
    raise DeadWorker
  end

  result = begin
    @serializer.load(read)
  rescue EOFError
    raise DeadWorker
  end
  raise result.exception if result.is_a?(ExceptionWrapper)
  result
end